function [A0,A,B,Z,Cx,Cu,Q,U,R] = calc_matrix(n,n3,ny,nuu,betta,rho_a,rho_u,sigma_nu,kappa_nu,lambda_x)

% Model description
% A_0*[X1(t+1) E X2(t+1)]' = A_1*[X1(t) X2(t)]' + B*u(t) + C*Eps(t+1)
% X1: # of predetermined variables, X2: # of jump variables
% u : # of instruments both policymaker and evil agent
%  Eps: Vector of shocks
 
% g, u, y, pi
A0 = zeros(n,n);
A0(1,1) = 1;
A0(2,2) = 1;
A0(3,3) = 1;
A0(4,4) = sigma_nu; A0(4,5) = 1;
A0(5,5) = betta;
  
A1 = zeros(n,n);
A1(1,1) = rho_a; A1(2,2) = rho_u; 
A1(3,4) = 1; 
A1(4,1) = -1; A1(4,4) = sigma_nu;
A1(5,2) = -1; A1(5,4) = -kappa_nu; A1(5,5) = 1;
  
B0 = zeros(n,n3);
B0(4,1) = 1;
  
% Reshape the above matrices to solve optimal monetary policy with robust control
% Thus, x(t+1) = A*x(t) + B*u
A = A0\A1;
B = A0\B0;
%tilC = A0\C;

% weight in social loss function
Z = diag([lambda_x 1]);

Cx = zeros(ny,n);
      Cx(1,4) = 1; % stabilization of output gap
      Cx(2,5) = 1;
Cu = zeros(ny,nuu);

Q = Cx'*Z*Cx;
U = Cx'*Z*Cu;
R = Cu'*Z*Cu;

end
