
% Robust control and optimal monetary policy inertia
% in a small-open new Keynesian model

% Daisuke Ida and Mitsuhiro Okano

% December 11, 2021

% housekeeping

clear
close

setpath_io

% # of variables
n1 = 3; % # of predetermined variables
n2 = 2; % # of jump variables
n3 = 1; % # of instrument variables
n4 = 2; % # of shock variables

% Target variables: Y(t) = Cx x(t) + Cu u(t)
ny = 2; % # of target variables in the loss function
nuu = 1; % # of instrument variables in the loss function

n = n1+n2;

% parameters
params; % call param.m

%theta_vec = [1000 750 500 400 300 200 100];
%num_tht = size(theta_vec,2);
theta_evil = 100;

% variance-covariance matrix
SigmaXX = zeros(n1,n1); SigmaXX(1,1) = sdev_a^2; SigmaXX(2,2) = sdev_u^2;

% weight on true loss function
lambda_x = 0.25;

% calculate structural parameters
[var_omega,frac_phi,Phi,sigma_nu,zetta,kappa_nu] = calc_deep_param(sigma,varphi,eta,theta,alpha,nu,betta);

% calculate structural matrices
[A0,A,B,Z,Cx,Cu,Q,U,R] = calc_matrix(n,n3,ny,nuu,betta,rho_a,rho_u,sigma_nu,kappa_nu,lambda_x);

% Robust approach
C  =  [ zeros(n1); ...
            zeros(n2,n1) ];
C(1,1) = 1; C(2,2) = 1;

% shock vector
uy  = A0\[sdev_a;0;0;0;0];  % positive demand shock
upi = A0\[0;sdev_u;0;0;0];  % mark-up shock

% for commitment
upi = upi(1:n1);
uy  = uy(1:n1);

% for discretion
C1=C(1:n1,:);
uyy = uy(1:n1); upii = upi(1:n1);

% option for commitment
cutoff = 1.0;

% main code (1): calculating welfare loss
call_policy_single; % call_policy_single.m

% main code (2): calculating impulse response function
Tbig = 15; % # periods

call_irf;

