function [M,N,Ma,Na,J0,J0a,FvX] = SimpAlgROpt( Fv0,A,B,Q,R,U,bet,n1,n2,Fu,cutoff,C,theta );
%  SimpAlgROpt   Solving robust RE model with simple policy rule,
%                optimizating evil agent's rule. Uses mdsmax in Octave 2.1.37+
%                or fminsearch in MatLab to optimize SimpAlgRLoss wrt Fv.
%
%
%
%  Usage:    [M,N,Ma,Na,J0,J0a,FvX] = SimpAlgROpt( Fv0,A,B,Q,R,U,bet,n1,n2,Fu,cutoff,C,theta );
%
%
%  Input:    Fv0     n1x(n1+n2) matrix, initial guesses of the evil agent's
%                    decision rule v = -Fv*x. The last n2 columns should be
%                    be zero.
%            A, B, Q, R, U, bet, n1, n2, Fu, cutoff, C, theta: see SimpAlgR
%
%
%  Output:   M        see SimpAlgR
%            N
%            Ma
%            Na
%            J0
%            J0a
%            FvX      n1xn matrix, optimal decision rule
%
%
%  Note:     In MatLab. this function could use fminsearch for optimization.
%            In Octave, it could use mdsmax from the Matrix Computation toolbox, available
%            (free) from http://www.maths.man.ac.uk/~higham/mctoolbox/
%
%
%  Callson:  mdsmax (if Octave)
%
%
%
%  Paolo Giordani and Paul.Soderlind@unisg.ch, Sep 2001
%------------------------------------------------------------------------------

par0 = Fv0(:,1:n1);      %mdsmax wants a vector as argument
par0 = par0(:);          %same as vec(Fv0(:,1:n1))
                                                      %finding optimal Fv
if exist('OCTAVE_VERSION');                  %mdsmax maximizes, good at discontinuities
  [parX,fval,nf] = mdsmax('SimpAlgRLoss',par0,1e-5,[], ...
                          A,B,Q,R,U,bet,n1,n2,Fu,cutoff,C,theta);
else;
  options = optimset('Display','iter');      %fminsearch minimizes, good at discontinuities
  [parX,fval,exitflag,output] = fminsearch('SimpAlgRLoss',par0,options, ...
                                            A,B,Q,R,U,bet,n1,n2,Fu,cutoff,C,theta);
end;

FvX = [ reshape(parX,n1,n1), zeros(n1,n2) ];          %n1^2 vector -> n1xn matrix
disp(' ');disp('argmax from fmincon is');disp( FvX );

                                                      %implications of optimal Fv
[M,N,Ma,Na,J0,J0a] = SimpAlgR(A,B,Q,R,U,bet,n1,n2,Fu,zeros(n1,1),cutoff,C,theta,FvX,1);
%------------------------------------------------------------------------------
