function [M,C,V,F] = solve_omp(A,B,Q,R,U,bet,n1,n2,cutoff,Vt1,Ct1,...
                              ConvCrit,Vweight,Fweight,CritLags,step,PrintIt,MaxIter,polregime)
%
%  Combine function of Paul Soerlind's, ComItAlg and DiscAlg.
%
%  (I change the section of Blanchard and Kahn.
%   I use MATLAB routine ORDQZ, instead of REORDER.m.)
%

%
% Discretion
%
if polregime == 0
Q = (Q + Q')/2;                %to make symmetric
R = (R + R')/2;

%n = n1 + n2;
Ft1 = 1000;

Cdiff = 1000*ones(1+CritLags,2);
iter = 1;
while any( max(Cdiff) > (ConvCrit')) && (iter < MaxIter);   %iterations

  n = n1 + n2;

A11 = A(1:n1,1:n1);
A12 = A(1:n1,(n1+1):n);
A21 = A((n1+1):n,1:n1);
A22 = A((n1+1):n,(n1+1):n);
Q11 = Q(1:n1,1:n1);
Q12 = Q(1:n1,(n1+1):n);
Q21 = Q((n1+1):n,1:n1);
Q22 = Q((n1+1):n,(n1+1):n);

B1 = B(1:n1,:);
B2 = B(n1+1:n,:);
U1 = U(1:n1,:);
U2 = U(n1+1:n,:);


D = (A22-Ct1*A12)\(Ct1*A11-A21);
G = (A22-Ct1*A12)\(Ct1*B1-B2);

As = A11 + A12*D;
Bs = B1 + A12*G;

Qs = Q11 + Q12*D + D'*Q21 + D'*Q22*D;
Us = Q12*G + D'*Q22*G + U1 + D'*U2;
Rs = R + G'*Q22*G + G'*U2 + U2'*G;

F = (Rs + bet*Bs'*Vt1*Bs)\(Us' + bet*Bs'*Vt1*As); %u(t) = -F*x1(t)
V = Qs - Us*F - F'*Us' + F'*Rs*F + bet*(As-Bs*F)'*Vt1*(As-Bs*F);

C = D - G*F;                   %x2(t)=C*x1(t)
M = A11 + A12*C - B1*F;        %x1(t+1) = M*x1(t) + e(t+1)

  Vdiff = max(  max( Vweight.*abs(V-Vt1) )  );        %changes t+1 -> t
  Fdiff = max(  max( Fweight.*abs(F-Ft1) )  );
  Cdiff2 = [Cdiff;Vdiff, Fdiff ];
  Cdiff = Cdiff2(2:size(Cdiff2,1),:);                   %latest is last


  Vt1 = step*V + (1-step)*Vt1;                        %"downdating"
  Ct1 = C;
  Ft1 = step*F + (1-step)*Ft1;


  if PrintIt == 1;
    disp(iter);
  elseif PrintIt == 2;
    disp([iter, (max(Cdiff))]);
  end;

  iter = iter + 1;

end;                                 %end iterations

if iter >= MaxIter;
  warning('Maximum number of iterations reached');
end;
end; % regime discretion


if polregime == 1
Q = (Q + Q')/2;                %to make symmetric
R = (R + R')/2;

n = n1 + n2;
k = size(R,1);                %No. of control variables

G =  [ eye(n),       zeros(n,k),   zeros(n,n)   ;
       zeros(n,n),   zeros(n,k),   (bet*A')     ;
       zeros(k,n),   zeros(k,k),   (-B')        ];

D =  [ A,            B,            zeros(n,n)   ;
       (-bet*Q),     (-bet*U),     eye(n)       ;
       U',           R,            zeros(k,n)   ];


G11 = G(:,1:n1);
G12 = G(:,n1+1:n1+n2);
G13 = G(:,n1+n2+1:n1+n2+k+n1);
G14 = G(:,n1+n2+k+n1+1:n1+n2+k+n1+n2);
D11 = D(:,1:n1);
D12 = D(:,n1+1:n1+n2);
D13 = D(:,n1+n2+1:n1+n2+k+n1);
D14 = D(:,n1+n2+k+n1+1:n1+n2+k+n1+n2);


G = [ G11, G14, G12, G13 ];          %x1,x2,(u,p1),p2 -> x1,p2,x2,(u,p1)
D = [ D11, D14, D12, D13 ];


  [S,T,Qa,Z] = qz(G,D);    %MatLab: G=Q'SZ' and D=Q'TZ'; Paul S:  G=QSZ' and D=QTZ', but Q isn't used
  %[S,T,~,Z] = reorder(S,T,Qa,Z);   % reordering of generalized eigenvalues, T(i,i)/S(i,i), in ascending order
  s = diag(S);
  t = diag(T);
  lambda = abs(t./s) < 1-eps;
  [S,T,~,Z] = ordqz(S,T,Qa,Z,lambda);
  logcon = abs(diag(T)) <= (abs(diag(S))*cutoff);   %1 for stable eigenvalue

if sum(logcon) < n;
  warning('Too few stable roots: no stable solution');
  M = NaN; C = NaN;
  return;
elseif sum(logcon) > n;
  warning('Too many stable roots: inifite number of stable solutions');
  M = NaN; C = NaN;
  return;
end;

Stt = S(1:n,1:n);
Ttt = T(1:n,1:n);
Zkt = Z(1:n,1:n);
Zlt = Z(n+1:n+k+n,1:n);

if cond(Zkt) > 1e+14;
  warning('Zkt is singular: rank condition for solution not satisfied');
  M = NaN; C = NaN;
  return;
end;

Zkt_1 = Zkt\eye(size(Zkt));         %inverting
Stt_1 = Stt\eye(size(Stt));


M = real(Zkt*Stt_1*Ttt*Zkt_1);      %[x1(t+1),p2(t+1)] = M*[x1(t),p2(t)]+e(t+1)
C = real(Zlt*Zkt_1);                %[x2(t),u(t),p1(t)] =C*[x1(t),p2(t)]
end
%-----------------------------------------------------------------------

