function [Mw,NMw,Ma,Na,Fv,J0,J0a] =solve_omp_r(A,B,Q,R,U,bet,n1,n2,cutoff,Vt1,Ct1,...
                              ConvCrit,Vweight,Fweight,CritLags,step,PrintIt,MaxIter,polregime,C,theta)
          
if polregime == 0 % discretion

   
Q = (Q + Q')/2;                %to make symmetric
R = (R + R')/2;

k = size(R,1);
n = n1 + n2;

if any(any(C(n1+1:n,:)));
  warning('Non-zero elements found in the last n2 rows of C');
  return;
end;


Btilde = [ B, C ];
Utilde = [ U, zeros(n1+n2,n1) ];
Rtilde = [ R,            zeros(k,n1); ...
           zeros(n1,k),  -theta*eye(n1) ];


[M,N,V,F] = DiscAlg( A,Btilde,Q,Rtilde,Utilde,bet,n1,n2,Vt1,Ct1, ...
                     ConvCrit,Vweight,Fweight,CritLags,step,PrintIt,MaxIter );

A11 = A(1:n1,1:n1);
A12 = A(1:n1,(n1+1):n);
A21 = A((n1+1):n,1:n1);
A22 = A((n1+1):n,(n1+1):n);

B1 = B(1:n1,:);
B2 = B(n1+1:n,:);

C1 = C(1:n1,:);
C2 = C(n1+1:n,:);

Fu = F(1:k,:);
Fv = F(k+1:k+n1,:);

Mw  = A11 + A12*N - B1*Fu - C1*Fv;
NMw = A21 + A22*N - B2*Fu;


Ma  = A11 + A12*N - B1*Fu;
Na  = N;


Ptilde = [ eye(n1); N; -Fu; -Fv ];             %value function, worst case
W      = Ptilde' * [Q, Utilde; Utilde', Rtilde] * Ptilde;
vecV   = (eye(n1*n1) - kron(M',bet*M'))\W(:);
V      = reshape(vecV,n1,n1);                 %solves V = W + bet*M'*V*M
if bet == 1;
  J0 = trace(C1'*V*C1*eye(n1));
else;
  J0 = bet/(1-bet) * trace(C1'*V*C1*eye(n1));
end;


Ptilde = [ eye(n1); Na; -Fu; zeros(n1,n1) ];             %value function
W      = Ptilde' * [Q, Utilde; Utilde', Rtilde] * Ptilde;
vecV   = (eye(n1*n1) - kron(Ma',bet*Ma'))\W(:);
V      = reshape(vecV,n1,n1);                 %solves V = W + bet*Ma'*V*Ma
if bet == 1;
  J0a = trace(C1'*V*C1*eye(n1));
else;
  J0a = bet/(1-bet) * trace(C1'*V*C1*eye(n1));
end;
    
end
    
if polregime == 1
    

Q = (Q + Q')/2;                %to make symmetric
R = (R + R')/2;

n1 = size(A,1);                %no of variables (all stable)
k  = size(R,1);                %no. of control variables


Btilde = [ B, C ];
Utilde = [ U, zeros(n1,n1) ];
Rtilde = [ R,            zeros(k,n1); ...
           zeros(n1,k),  -theta*eye(n1) ];


[M,N] = ComAlgS( A,Btilde,Q,Rtilde,Utilde,bet,cutoff );   %commitment, robust policy


N2 = N(1:k,:);                        % u = N2*x1; note: the P matrix is eye(n1) since n2=0
N3 = N(k+1:k+n1,:);                   % v = N3*x1

Fu  = -N2;                       %u = -Fu*x1
Fv  = -N3;                       %v = -Fv*x1

Mw = A-B*Fu-C*Fv;     %alternative expression for M, should equal M
%disp(' ');disp('M-Mw');disp([M-Mw]);


Ma = A-B*Fu;                     %x1(t+1) = Ma*x1(t), approximating model
Na = N;                          %[u;v;p1] = Na*x1, approximating model
Na(k+1:k+n1,:) = zeros(n1,n1);   %no evil shocks in approximate model

maxEig = max(abs(eig(Ma)));
if maxEig > cutoff;
  warning('Some abs(eigenvalue) of Ma > 1');
end;
%disp(' ');disp('M-Ma');disp([M-Ma]);



Ptilde = [ eye(n1); -Fu; -Fv ];             %value function, worst case
W      = Ptilde' * [Q, Utilde; Utilde', Rtilde] * Ptilde;
vecV   = (eye(n1*n1) - kron(M',bet*M'))\W(:);
V      = reshape(vecV,n1,n1);                 %solves V = W + bet*M'*V*M
if bet == 1;
  J0 = trace(C'*V*C*eye(n1));
else;
  J0 = bet/(1-bet) * trace(C'*V*C*eye(n1));
end;


Ptilde = [ eye(n1); -Fu; zeros(n1,n1) ];             %value function
W      = Ptilde' * [Q, Utilde; Utilde', Rtilde] * Ptilde;
vecV   = (eye(n1*n1) - kron(Ma',bet*Ma'))\W(:);
V      = reshape(vecV,n1,n1);                 %solves V = W + bet*Ma'*V*Ma
if bet == 1;
  J0a = trace(C'*V*C*eye(n1));
else;
  J0a = bet/(1-bet) * trace(C'*V*C*eye(n1));
end;

end % end of commitment 
                         
                       
end